import json
import pika
import time
from loguru import logger


class RabbitMQProducer:
    def __init__(self, host: str, port: int, exchange: str, exchange_type: str = 'fanout'):
        self.host = host
        self.port = port
        self.exchange = exchange
        self.exchange_type = exchange_type

        self.max_retries = 5
        self.initial_delay = 1  # Initial delay in seconds for exponential backoff

        self.connection = None
        self.channel = None
        self.connect()

    def connect(self):
        retries = 0
        while retries < self.max_retries:
            try:
                self.connection = pika.BlockingConnection(pika.ConnectionParameters(host=self.host, port=self.port))
                self.channel = self.connection.channel()
                self.channel.exchange_declare(exchange=self.exchange, exchange_type=self.exchange_type)
                logger.info(f"Successfully connected to RabbitMQ at {self.host}:{self.port}")
                return
            except Exception as e:
                retries += 1
                wait_time = self.initial_delay * (2 ** (retries - 1))
                logger.warning(f"Could not connect to RabbitMQ (attempt {retries}/{self.max_retries}): {e}")
                if retries < self.max_retries:
                    logger.info(f"Retrying in {wait_time} seconds...")
                    time.sleep(wait_time)
                else:
                    logger.error("All connection attempts failed.")
                    raise

    def send_event(self, message: dict):
        retries = 0

        while retries < self.max_retries:
            if self.connection is None or not self.connection.is_open:
                logger.info("Connection is closed. Reconnecting...")
                self.connect()

            try:
                self.channel.basic_publish(
                    exchange=self.exchange,
                    routing_key='',  # Empty routing key for fanout exchange
                    body=json.dumps(message),
                    properties=pika.BasicProperties(content_type='application/json')
                )
                logger.info(f"Message sent: {message}")
                return
            except pika.exceptions.AMQPConnectionError as e:
                retries += 1
                wait_time = self.initial_delay * (2 ** (retries - 1))
                logger.warning(f"Connection error (attempt {retries}/{self.max_retries}): {e}")
                if retries < self.max_retries:
                    logger.info(f"Retrying in {wait_time} seconds...")
                    self.connect()
                    time.sleep(wait_time)
                else:
                    logger.error("All retry attempts failed.")
                    raise
            except Exception as e:
                logger.warning(f"Failed to send message: {e}")
                raise

    def close_connection(self):
        if self.connection and self.connection.is_open:
            self.connection.close()


# Usage example:
if __name__ == "__main__":
    producer = RabbitMQProducer(host='localhost', port=5672, exchange='logs')
    try:
        producer.send_event({"message": "Hello, RabbitMQ!"})
    finally:
        producer.close_connection()
